## Copyright (C) 2009-2014   Lukas F. Reichlin
##
## This file is part of LTI Syncope.
##
## LTI Syncope is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## -*- texinfo -*-
## Frequency response of SS models.

## Author: Lukas Reichlin <lukas.reichlin@gmail.com>
## Created: October 2009
## Version: 0.6

## More robust version (eig): Marek Peca <hefaistos@gmail.com> Oct 2015

function H = ssfreqresp(sys, w, cellflag = false)

  if (sys.scaled == false)
    sys = prescale (sys);
  endif

  [a, b, c, d, e, tsam] = dssdata (sys);

  if (isct (sys))  # continuous system
    s = i * w;
  else             # discrete system
    s = exp (i * w * abs (tsam));
  endif

  [V,D]=eig(a);  % TODO: only e=eye supported!
  D=diag(D);
  B2=inv(V)*b;
  C2=c*V;

  % FIXME: for stiff systems (e != identity) either
  % a) add support
  % b) revert to old behaviour:
  %    H = arrayfun (@(x) c/(x*e - a)*b + d, s, "uniformoutput", false);
  
  H = arrayfun (@(x) C2*diag(1./(x-D))*B2 + d, s, "uniformoutput", false);

  if (! cellflag)
    H = cat (3, H{:});
  endif

endfunction
